"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require("../");
const rx = require("rx");
const fs = require("fs");
const zlib = require("zlib");
const path = require("path");
const log4js = require("log4js");
const logger = log4js.getLogger('spiderjs');
class Project {
    constructor(projectpath) {
        this.projectpath = projectpath;
    }
    load() {
        const configfile = path.join(this.projectpath, 'spiderjs.json');
        if (!fs.existsSync(configfile)) {
            logger.error(`can't find spiderjs.json file in directory: ${this.projectpath}`);
            return rx.Observable.empty();
        }
        try {
            this.spiderjs = JSON.parse(fs.readFileSync(configfile, 'utf-8'));
            const rootpath = path.join(this.projectpath, 'src');
            const app = this.spiderjs.app;
            app.lambda = [];
            fs.readdirSync(rootpath)
                .forEach((script) => {
                if (script !== 'main.js' && path.extname(script) === '.js') {
                    const data = zlib.gzipSync(fs.readFileSync(path.join(rootpath, script))).toString('base64');
                    const lambda = {
                        name: script,
                        script: data,
                    };
                    app.lambda.push(lambda);
                }
            });
            return rx.Observable.just(this.spiderjs);
        }
        catch (error) {
            return rx.Observable.throw(error);
        }
    }
    save() {
        const lambda = this.spiderjs.app.lambda;
        this.spiderjs.app.lambda = undefined;
        return rx.Observable.create((ob) => {
            fs.writeFile(path.join(this.projectpath, 'spiderjs.json'), JSON.stringify(this.spiderjs, undefined, '\t'), (error) => {
                this.spiderjs.app.lambda = lambda;
                if (error) {
                    ob.onError(error);
                }
                else {
                    ob.onNext({});
                    ob.onCompleted();
                }
            });
        });
    }
    deploy() {
        return this.getEngine()
            .flatMap((engine) => {
            logger.info(`create app ...`);
            return engine.createApp(this.spiderjs.app);
        })
            .flatMap((oid) => {
            logger.info(`create app -- success`, oid);
            this.spiderjs.app.oid = oid;
            logger.info(`create exeuctor for accesskey`, this.spiderjs.deploy);
            return this
                .getEngine()
                .flatMap((engine) => {
                return engine.deploy(oid, this.spiderjs.deploy);
            });
        })
            .flatMap((oid) => {
            logger.info(`create exeuctor for accesskey`, this.spiderjs.deploy, oid);
            this.spiderjs.executor = oid;
            return this.save().map(() => oid);
        });
    }
    undeploy() {
        if (this.spiderjs.executor) {
            return this
                .getEngine()
                .flatMap((engine) => {
                return engine
                    .undeploy(this.spiderjs.executor)
                    .map(() => {
                    logger.error(`undeploy executor[${this.spiderjs.executor}] -- success`);
                    return engine;
                })
                    .catch((error) => {
                    logger.error(`undeploy executor[${this.spiderjs.executor}] -- failed`);
                    return rx.Observable.just(engine);
                });
            })
                .flatMap((engine) => {
                this.spiderjs.executor = undefined;
                if (this.spiderjs.app.oid) {
                    return engine
                        .deleteApp(this.spiderjs.app.oid)
                        .map(() => {
                        logger.error(`undeploy app[${this.spiderjs.app.oid}] -- success`);
                        return {};
                    })
                        .catch((error) => {
                        logger.error(`undeploy app[${this.spiderjs.app.oid}] -- failed`);
                        return rx.Observable.just({});
                    });
                }
                return rx.Observable.just({});
            })
                .flatMap(() => {
                return this.save();
            });
        }
        return rx.Observable.just({});
    }
    createTask(name) {
        if (!this.spiderjs.task) {
            this.spiderjs.task = {};
        }
        if (this.spiderjs.task[name]) {
            return rx.Observable.throw(new Error(`task[${name}] already exists`));
        }
        return this
            .getEngine()
            .flatMap((engine) => {
            return engine.createTask(this.spiderjs.executor);
        })
            .flatMap((oid) => {
            this.spiderjs.task[name] = oid;
            return this.save().map(() => oid);
        });
    }
    getTask(name) {
        if (!this.spiderjs.task) {
            this.spiderjs.task = {};
        }
        if (!this.spiderjs.task[name]) {
            return rx.Observable.throw(new Error(`can't find task[${name}]`));
        }
        return this
            .getEngine()
            .flatMap((engine) => {
            return engine.getTask(this.spiderjs.task[name]);
        });
    }
    closeTask(name) {
        if (!this.spiderjs.task) {
            this.spiderjs.task = {};
        }
        if (!this.spiderjs.task[name]) {
            return rx.Observable.throw(new Error(`can't find task[${name}]`));
        }
        return this
            .getEngine()
            .flatMap((engine) => {
            return engine.closeTask(this.spiderjs.task[name]);
        })
            .flatMap((oid) => {
            this.spiderjs.task[name] = undefined;
            return this.save();
        });
    }
    getEngine() {
        if (!this.engine) {
            this.engine = new _1.Engine(this.spiderjs.url, this.spiderjs.accesskey);
            return this.engine.connect();
        }
        return rx.Observable.just(this.engine);
    }
}
exports.default = Project;
//# sourceMappingURL=project.js.map