import {
    IAccessKey, IApp, ICall, IEngine, IExecutor, ITask,
} from './api';

import rx = require('rx');
import log4js = require('log4js');
import request = require('request');
const logger = log4js.getLogger('spiderjs-sdk-admin');

export class Engine implements IEngine {
    private token: string;
    constructor(private url: string, private accesskey: IAccessKey) {

    }

    public connect(): rx.Observable<IEngine> {
        return this
            .post<string>('/admin/accesskey', { oid: this.accesskey.oid, key: this.accesskey.key })
            .map((token) => {
                this.token = token;
                return this;
            });
    }

    public createApp(app: IApp): rx.Observable<string> {
        return this.post<string>('/engine/app', { app });
    }

    public deleteApp(oid: string): rx.Observable<{}> {
        return this.delete<string>('/engine/app', { oid });
    }

    public getApp(oid: string): rx.Observable<IApp> {
        return this.get<IApp>('/engine/app', { oid });
    }

    public deploy(oid: string, accesskey: string): rx.Observable<string> {
        return this.post<string>('/engine/executor', { oid, accesskey });
    }

    public undeploy(oid: string): rx.Observable<{}> {
        return this.delete<string>('/engine/executor', { oid });
    }

    public getExecutor(oid: string): rx.Observable<IExecutor> {
        return this.get<IExecutor>('/engine/executor', { oid });
    }

    public createTask(oid: string): rx.Observable<string> {
        return this.post<string>('/engine/task', { oid });
    }

    public closeTask(oid: string): rx.Observable<{}> {
        return this.delete('/engine/task', { oid });
    }

    public getTask(oid: string): rx.Observable<ITask> {
        return this.get<ITask>('/engine/task', { oid });
    }

    public call(call: ICall): rx.Observable<ITask> {
        return this.post<ITask>('/engine/call', { call });
    }

    private post<T>(method: string, params?: any): rx.Observable<T> {

        const headers: any = {};

        headers.Authorization = `JWT ${this.token}`;

        return rx.Observable.create<T>((ob) => {
            request.post({
                body: params,
                headers,
                json: true,
                timeout: 6000,
                url: `${this.url}${method}`,
            }, (error, response, resp) => {
                if (error) {
                    ob.onError(error);
                    return;
                }

                if (resp.code !== 'SUCCESS') {
                    ob.onError(resp);
                    return;
                }

                ob.onNext(resp.data);

                ob.onCompleted();
            });
        });
    }

    private delete<T>(method: string, params?: any): rx.Observable<T> {

        const headers: any = {};

        headers.Authorization = `JWT ${this.token}`;

        return rx.Observable.create<T>((ob) => {
            request.del({
                body: params,
                headers,
                json: true,
                url: `${this.url}${method}`,
            }, (error, response, resp) => {
                if (error) {
                    ob.onError(error);
                    return;
                }

                if (resp.code !== 'SUCCESS') {
                    ob.onError(resp);
                    return;
                }

                ob.onNext(resp.data);

                ob.onCompleted();
            });
        });
    }

    private get<T>(method: string, params?: any): rx.Observable<T> {

        const headers: any = {};

        headers.Authorization = `JWT ${this.token}`;

        return rx.Observable.create<T>((ob) => {
            request.get({
                headers,
                json: true,
                qs: params,
                url: `${this.url}${method}`,
            }, (error, response, resp) => {
                if (error) {
                    ob.onError(error);
                    return;
                }

                if (resp.code !== 'SUCCESS') {
                    ob.onError(resp);
                    return;
                }

                ob.onNext(resp.data);

                ob.onCompleted();
            });
        });
    }
}
